@@ -272,18 +272,24 @@ class Installer:
         """ Try MongoDB configuration """
         try:
             mongo_client = MongoClient(host=host)
+            # Effective access only occurs when we call a method on the connexion
+            mongo_version = str(mongo_client.server_info()['version'])
+            self._display_info("Found mongodb server running version %s on %s." % (mongo_version, host))
         except Exception as e:
             self._display_warning("Cannot connect to MongoDB on host %s: %s" % (host, str(e)))
             return None
 
         try:
             database = mongo_client[database_name]
+            # Effective access only occurs when we call a method on the database.
+            database.list_collection_names()
         except Exception as e:
             self._display_warning("Cannot access database %s: %s" % (database_name, str(e)))
             return None
 
         try:
-            GridFS(database)
+            # Effective access only occurs when we call a method on the gridfs object.
+            GridFS(database).find_one()
         except Exception as e:
             self._display_warning("Cannot access gridfs %s: %s" % (database_name, str(e)))
             return None
@@ -376,7 +382,7 @@ class Installer:
             for image in to_download:
                 try:
                     self._display_info("Downloading image %s. This can take some time." % image)
-                    docker_connection.images.pull(image + ":v0.5")
+                    docker_connection.images.pull(image + ":v0.6")
                 except Exception as e:
                     self._display_error("An error occurred while pulling the image: %s." % str(e))
         else:
