@@ -16,6 +16,7 @@ import time
 import web
 from bson.objectid import ObjectId
 from collections import OrderedDict
+from pymongo import ReturnDocument
 
 from inginious.common import exceptions
 from inginious.frontend.pages.utils import INGIniousPage
@@ -101,8 +102,8 @@ class BaseTaskPage(object):
         try:
             tasks = OrderedDict((tid, t) for tid, t in course.get_tasks().items() if self.user_manager.task_is_visible_by_user(t, username, isLTI))
             task = tasks[taskid]
-        except exceptions.TaskNotFoundException as ex:
-            raise web.notfound(str(ex))
+        except KeyError:
+            raise web.notfound()
 
         if not self.user_manager.task_is_visible_by_user(task, username, isLTI):
             return self.template_helper.get_renderer().task_unavailable()
@@ -116,15 +117,6 @@ class BaseTaskPage(object):
         self.user_manager.user_saw_task(username, courseid, taskid)
 
         is_staff = self.user_manager.has_staff_rights_on_course(course, username)
-        
-        # Generate random inputs and save it into db
-        random_input_list = []
-        random.seed(str(username if username is not None else "") + taskid + courseid + str(time.time() if task.regenerate_input_random() else ""))
-        random_input_list = [random.random() for i in range(task.get_number_input_random())]
-        if username is not None and len(random_input_list) > 0: # Avoid errors when visitors/unregistered users and avoid pollute db if input random not using
-            self.database.user_tasks.update(
-                    {"courseid": task.get_course_id(), "taskid": task.get_id(), "username": username},
-                    {"$set": {"random": random_input_list}})
 
         userinput = web.input()
         if "submissionid" in userinput and "questionid" in userinput:
@@ -147,12 +139,22 @@ class BaseTaskPage(object):
                 web.header('Content-Type', 'text/plain')
                 return sinput[userinput["questionid"]]
         else:
-            # user_task always exists as we called user_saw_task before
-            user_task = self.database.user_tasks.find_one({
-                "courseid": task.get_course_id(),
-                "taskid": task.get_id(),
-                "username": self.user_manager.session_username()
-            })
+            # Generate random inputs and save it into db
+            random.seed(str(username if username is not None else "") + taskid + courseid + str(
+                time.time() if task.regenerate_input_random() else ""))
+            random_input_list = [random.random() for i in range(task.get_number_input_random())]
+
+            user_task = self.database.user_tasks.find_one_and_update(
+                {
+                    "courseid": task.get_course_id(),
+                    "taskid": task.get_id(),
+                    "username": self.user_manager.session_username()
+                },
+                {
+                    "$set": {"random": random_input_list}
+                },
+                return_document=ReturnDocument.AFTER
+            )
 
             submissionid = user_task.get('submissionid', None)
             eval_submission = self.database.submissions.find_one({'_id': ObjectId(submissionid)}) if submissionid else None
@@ -176,124 +178,122 @@ class BaseTaskPage(object):
     def POST(self, courseid, taskid, isLTI):
         """ POST a new submission """
         username = self.user_manager.session_username()
-        try:
-            course = self.course_factory.get_course(courseid)
-            if not self.user_manager.course_is_open_to_user(course, username, isLTI):
-                return self.template_helper.get_renderer().course_unavailable()
 
-            task = course.get_task(taskid)
-            if not self.user_manager.task_is_visible_by_user(task, username, isLTI):
-                return self.template_helper.get_renderer().task_unavailable()
+        course = self.course_factory.get_course(courseid)
+        if not self.user_manager.course_is_open_to_user(course, username, isLTI):
+            return self.template_helper.get_renderer().course_unavailable()
 
-            self.user_manager.user_saw_task(username, courseid, taskid)
+        task = course.get_task(taskid)
+        if not self.user_manager.task_is_visible_by_user(task, username, isLTI):
+            return self.template_helper.get_renderer().task_unavailable()
 
-            is_staff = self.user_manager.has_staff_rights_on_course(course, username)
-            is_admin = self.user_manager.has_admin_rights_on_course(course, username)
+        self.user_manager.user_saw_task(username, courseid, taskid)
 
-            userinput = web.input()
-            if "@action" in userinput and userinput["@action"] == "submit":
-                # Verify rights
-                if not self.user_manager.task_can_user_submit(task, username, isLTI):
-                    return json.dumps({"status": "error", "text": _("You are not allowed to submit for this task.")})
-                    
-                # Retrieve input random and check still valid
-                random_input = self.database.user_tasks.find_one({"courseid": task.get_course_id(), "taskid": task.get_id(), "username": username}, { "random": 1 })
-                random_input = random_input["random"] if "random" in random_input else []
-                for i in range(0, len(random_input)):
-                    s = "@random_" + str(i)
-                    if s not in userinput or float(userinput[s]) != random_input[i]:
-                        return json.dumps({"status": "error", "text": _("Your task has been regenerated. This current task is outdated.")})
+        is_staff = self.user_manager.has_staff_rights_on_course(course, username)
+        is_admin = self.user_manager.has_admin_rights_on_course(course, username)
 
-                # Reparse user input with array for multiple choices
-                init_var = {
-                    problem.get_id(): problem.input_type()()
-                    for problem in task.get_problems() if problem.input_type() in [dict, list]
-                }
-                userinput = task.adapt_input_for_backend(web.input(**init_var))
+        userinput = web.input()
+        if "@action" in userinput and userinput["@action"] == "submit":
+            # Verify rights
+            if not self.user_manager.task_can_user_submit(task, username, isLTI):
+                return json.dumps({"status": "error", "text": _("You are not allowed to submit for this task.")})
 
-                if not task.input_is_consistent(userinput, self.default_allowed_file_extensions, self.default_max_file_size):
-                    web.header('Content-Type', 'application/json')
-                    return json.dumps({"status": "error", "text": _("Please answer to all the questions and verify the extensions of the files "
-                                                                  "you want to upload. Your responses were not tested.")})
-                del userinput['@action']
+            # Retrieve input random and check still valid
+            random_input = self.database.user_tasks.find_one({"courseid": task.get_course_id(), "taskid": task.get_id(), "username": username}, { "random": 1 })
+            random_input = random_input["random"] if "random" in random_input else []
+            for i in range(0, len(random_input)):
+                s = "@random_" + str(i)
+                if s not in userinput or float(userinput[s]) != random_input[i]:
+                    return json.dumps({"status": "error", "text": _("Your task has been regenerated. This current task is outdated.")})
 
-                # Get debug info if the current user is an admin
-                debug = is_admin
-                if "@debug-mode" in userinput:
-                    if userinput["@debug-mode"] == "ssh" and debug:
-                        debug = "ssh"
-                    del userinput['@debug-mode']
+            # Reparse user input with array for multiple choices
+            init_var = {
+                problem.get_id(): problem.input_type()()
+                for problem in task.get_problems() if problem.input_type() in [dict, list]
+            }
+            userinput = task.adapt_input_for_backend(web.input(**init_var))
 
-                # Start the submission
-                try:
-                    submissionid, oldsubids = self.submission_manager.add_job(task, userinput, debug)
-                    web.header('Content-Type', 'application/json')
-                    return json.dumps({"status": "ok", "submissionid": str(submissionid), "remove": oldsubids, "text": _("<b>Your submission has been sent...</b>")})
-                except Exception as ex:
-                    web.header('Content-Type', 'application/json')
-                    return json.dumps({"status": "error", "text": str(ex)})
+            if not task.input_is_consistent(userinput, self.default_allowed_file_extensions, self.default_max_file_size):
+                web.header('Content-Type', 'application/json')
+                return json.dumps({"status": "error", "text": _("Please answer to all the questions and verify the extensions of the files "
+                                                              "you want to upload. Your responses were not tested.")})
+            del userinput['@action']
 
-            elif "@action" in userinput and userinput["@action"] == "check" and "submissionid" in userinput:
-                result = self.submission_manager.get_submission(userinput['submissionid'], user_check=not is_staff)
-                if result is None:
-                    web.header('Content-Type', 'application/json')
+            # Get debug info if the current user is an admin
+            debug = is_admin
+            if "@debug-mode" in userinput:
+                if userinput["@debug-mode"] == "ssh" and debug:
+                    debug = "ssh"
+                del userinput['@debug-mode']
+
+            # Start the submission
+            try:
+                submissionid, oldsubids = self.submission_manager.add_job(task, userinput, debug)
+                web.header('Content-Type', 'application/json')
+                return json.dumps({"status": "ok", "submissionid": str(submissionid), "remove": oldsubids, "text": _("<b>Your submission has been sent...</b>")})
+            except Exception as ex:
+                web.header('Content-Type', 'application/json')
+                return json.dumps({"status": "error", "text": str(ex)})
+
+        elif "@action" in userinput and userinput["@action"] == "check" and "submissionid" in userinput:
+            result = self.submission_manager.get_submission(userinput['submissionid'], user_check=not is_staff)
+            if result is None:
+                web.header('Content-Type', 'application/json')
+                return json.dumps({'status': "error", "text": _("Internal error")})
+            elif self.submission_manager.is_done(result, user_check=not is_staff):
+                web.header('Content-Type', 'application/json')
+                result = self.submission_manager.get_input_from_submission(result)
+                result = self.submission_manager.get_feedback_from_submission(result, show_everything=is_staff)
+
+                # user_task always exists as we called user_saw_task before
+                user_task = self.database.user_tasks.find_one({
+                    "courseid":task.get_course_id(),
+                    "taskid": task.get_id(),
+                    "username": {"$in": result["username"]}
+                })
+
+                default_submissionid = user_task.get('submissionid', None)
+                if default_submissionid is None:
+                    # This should never happen, as user_manager.update_user_stats is called whenever a submission is done.
                     return json.dumps({'status': "error", "text": _("Internal error")})
-                elif self.submission_manager.is_done(result, user_check=not is_staff):
-                    web.header('Content-Type', 'application/json')
-                    result = self.submission_manager.get_input_from_submission(result)
-                    result = self.submission_manager.get_feedback_from_submission(result, show_everything=is_staff)
 
-                    # user_task always exists as we called user_saw_task before
-                    user_task = self.database.user_tasks.find_one({
-                        "courseid":task.get_course_id(),
-                        "taskid": task.get_id(),
-                        "username": {"$in": result["username"]}
-                    })
-
-                    default_submissionid = user_task.get('submissionid', None)
-                    if default_submissionid is None:
-                        # This should never happen, as user_manager.update_user_stats is called whenever a submission is done.
-                        return json.dumps({'status': "error", "text": _("Internal error")})
-
-                    return self.submission_to_json(task, result, is_admin, False, default_submissionid == result['_id'], tags=task.get_tags())
-                else:
-                    web.header('Content-Type', 'application/json')
-                    return self.submission_to_json(task, result, is_admin)
-
-            elif "@action" in userinput and userinput["@action"] == "load_submission_input" and "submissionid" in userinput:
-                submission = self.submission_manager.get_submission(userinput["submissionid"], user_check=not is_staff)
-                submission = self.submission_manager.get_input_from_submission(submission)
-                submission = self.submission_manager.get_feedback_from_submission(submission, show_everything=is_staff)
-                if not submission:
-                    raise web.notfound()
-                web.header('Content-Type', 'application/json')
-                
-                return self.submission_to_json(task, submission, is_admin, True, tags=task.get_tags())
-                
-            elif "@action" in userinput and userinput["@action"] == "kill" and "submissionid" in userinput:
-                self.submission_manager.kill_running_submission(userinput["submissionid"])  # ignore return value
+                return self.submission_to_json(task, result, is_admin, False, default_submissionid == result['_id'], tags=course.get_tags())
+            else:
                 web.header('Content-Type', 'application/json')
+                return self.submission_to_json(task, result, is_admin, False, tags=course.get_tags())
+
+        elif "@action" in userinput and userinput["@action"] == "load_submission_input" and "submissionid" in userinput:
+            submission = self.submission_manager.get_submission(userinput["submissionid"], user_check=not is_staff)
+            submission = self.submission_manager.get_input_from_submission(submission)
+            submission = self.submission_manager.get_feedback_from_submission(submission, show_everything=is_staff)
+            if not submission:
+                raise web.notfound()
+            web.header('Content-Type', 'application/json')
+
+            return self.submission_to_json(task, submission, is_admin, True, tags=course.get_tags())
+
+        elif "@action" in userinput and userinput["@action"] == "kill" and "submissionid" in userinput:
+            self.submission_manager.kill_running_submission(userinput["submissionid"])  # ignore return value
+            web.header('Content-Type', 'application/json')
+            return json.dumps({'status': 'done'})
+        elif "@action" in userinput and userinput["@action"] == "set_submission" and "submissionid" in userinput:
+            web.header('Content-Type', 'application/json')
+            if task.get_evaluate() != 'student':
+                return json.dumps({'status': "error"})
+
+            if self.set_selected_submission(course, task, userinput["submissionid"]):
                 return json.dumps({'status': 'done'})
-            elif "@action" in userinput and userinput["@action"] == "set_submission" and "submissionid" in userinput:
-                web.header('Content-Type', 'application/json')
-                if task.get_evaluate() != 'student':
-                    return json.dumps({'status': "error"})
-
-                if self.set_selected_submission(course, task, userinput["submissionid"]):
-                    return json.dumps({'status': 'done'})
-                else:
-                    return json.dumps({'status': 'error'})
             else:
-                raise web.notfound()
-        except:
-            if web.config.debug:
-                raise
-            else:
-                raise web.notfound()
+                return json.dumps({'status': 'error'})
+        else:
+            raise web.notfound()
 
-    def submission_to_json(self, task, data, debug, reloading=False, replace=False, tags={}):
+    def submission_to_json(self, task, data, debug, reloading=False, replace=False, tags=None):
         """ Converts a submission to json (keeps only needed fields) """
 
+        if tags is None:
+            tags = []
+
         if "ssh_host" in data:
             return json.dumps({'status': "waiting", 'text': "<b>SSH server active</b>",
                                'ssh_host': data["ssh_host"], 'ssh_port': data["ssh_port"],
@@ -333,25 +333,28 @@ class BaseTaskPage(object):
             tojson["problems"] = data["problems"]
 
         if debug:
-            tojson["debug"] = data
+            tojson["debug"] = self._cut_long_chains(data)
 
         if tojson['status'] == 'waiting':
-            tojson["text"] = _("<b>Your submission has been sent...</b>")
+            tojson["title"] = _("<b>Your submission has been sent...</b>")
         elif tojson["result"] == "failed":
-            tojson["text"] = _("There are some errors in your answer. Your score is {score}%.").format(score=data["grade"])
+            tojson["title"] = _("There are some errors in your answer. Your score is {score}%.").format(score=data["grade"])
         elif tojson["result"] == "success":
-            tojson["text"] = _("Your answer passed the tests! Your score is {score}%.").format(score=data["grade"])
+            tojson["title"] = _("Your answer passed the tests! Your score is {score}%.").format(score=data["grade"])
         elif tojson["result"] == "timeout":
-            tojson["text"] = _("Your submission timed out. Your score is {score}%.").format(score=data["grade"])
+            tojson["title"] = _("Your submission timed out. Your score is {score}%.").format(score=data["grade"])
         elif tojson["result"] == "overflow":
-            tojson["text"] = _("Your submission made an overflow. Your score is {score}%.").format(score=data["grade"])
+            tojson["title"] = _("Your submission made an overflow. Your score is {score}%.").format(score=data["grade"])
         elif tojson["result"] == "killed":
-            tojson["text"] = _("Your submission was killed.")
+            tojson["title"] = _("Your submission was killed.")
         else:
-            tojson["text"] = _("An internal error occurred. Please retry later. "
-                               "If the error persists, send an email to the course administrator.")
+            tojson["title"] = _("An internal error occurred. Please retry later. "
+                                "If the error persists, send an email to the course administrator.")
 
-        tojson["text"] = "<b>" + tojson["text"] + " " + _("[Submission #{submissionid}]").format(submissionid=data["_id"]) + "</b>" + data.get("text", "")
+        tojson["title"] += " " + _("[Submission #{submissionid}]").format(submissionid=data["_id"])
+        tojson["title"] = self.plugin_manager.call_hook_recursive("feedback_title", task=task, submission=data, title=tojson["title"])["title"]
+        
+        tojson["text"] = data.get("text", "")
         tojson["text"] = self.plugin_manager.call_hook_recursive("feedback_text", task=task, submission=data, text=tojson["text"])["text"]
 
         if reloading:
@@ -362,10 +365,9 @@ class BaseTaskPage(object):
 
         if "tests" in data:
             tojson["tests"] = {}
-            if tags:
-                for tag in tags[0]+tags[1]: # Tags only visible for admins should not appear in the json for students.
-                    if (tag.is_visible_for_student() or debug) and tag.get_id() in data["tests"]:
-                        tojson["tests"][tag.get_id()] = data["tests"][tag.get_id()]
+            for key, tag in tags.items():  # Tags only visible for admins should not appear in the json for students.
+                if tag.get_type() in [0, 1] and (tag.is_visible_for_student() or debug) and key in data["tests"]:
+                    tojson["tests"][key] = data["tests"][key]
             if debug: #We add also auto tags when we are admin
                 for tag in data["tests"]:
                     if tag.startswith("*auto-tag-"):
@@ -376,6 +378,24 @@ class BaseTaskPage(object):
 
         return json.dumps(tojson, default=str)
 
+    def _cut_long_chains(self, data, limit=1000):
+        """ Cut all strings and byte chains in a dictionary
+            if they exceed a limit in characters or bytes """
+
+        if isinstance(data, dict):
+            new_data = {}
+            for key, value in data.items():
+                if isinstance(value, bytes) and len(value) > limit:
+                    new_data[key] = value[:limit] + _(" <text cut due to its length>").encode("utf-8")
+                elif isinstance(value, str) and len(value) > limit:
+                    new_data[key] = value[:limit] + _(" <text cut due to its length>")
+                elif isinstance(value, dict):
+                    new_data[key] = self._cut_long_chains(value)
+                else:
+                    new_data[key] = value
+            return new_data
+        return data
+
 
 class TaskPageStaticDownload(INGIniousPage):
     """ Allow to download files stored in the task folder """
@@ -391,13 +411,17 @@ class TaskPageStaticDownload(INGIniousPage):
             if not self.user_manager.course_is_open_to_user(course):
                 return self.template_helper.get_renderer().course_unavailable()
 
-            task = course.get_task(taskid)
-            if not self.user_manager.task_is_visible_by_user(task):  # ignore LTI check here
-                return self.template_helper.get_renderer().task_unavailable()
-
             path_norm = posixpath.normpath(urllib.parse.unquote(path))
 
-            public_folder = task.get_fs().from_subfolder("public")
+            if taskid == "$common":
+                public_folder = course.get_fs().from_subfolder("$common").from_subfolder("public")
+            else:
+
+                task = course.get_task(taskid)
+                if not self.user_manager.task_is_visible_by_user(task):  # ignore LTI check here
+                    return self.template_helper.get_renderer().task_unavailable()
+
+                public_folder = task.get_fs().from_subfolder("public")
             (method, mimetype_or_none, file_or_url) = public_folder.distribute(path_norm, False)
 
             if method == "local":
