@@ -64,7 +64,6 @@ function studio_update_file_tabs(data, method)
         data = {};
     if(method == undefined)
         method = "GET";
-
     jQuery.ajax({
         beforeSend: function()
                     {
@@ -145,9 +144,9 @@ function studio_task_file_open_tab(path)
         studio_file_editor_tabs[path] = tab_id;
 
         var edit_file_tabs = $('#edit_file_tabs');
-        edit_file_tabs.append('<li role="presentation" class="studio_file_editor_tab">' +
-            '<a href="#' + tab_id + '" aria-controls="editor" role="tab" data-toggle="tab"><i class="fa fa-file-code-o"></i>&nbsp; ' + path +
-            ' <button class="closetab" type="button"><i class="fa fa-remove"></i></button>' +
+        edit_file_tabs.append('<li class="nav-item studio_file_editor_tab">' +
+            '<a class="nav-link" href="#' + tab_id + '" aria-controls="editor" role="tab" data-toggle="tab"><i class="fa fa-file-code-o"></i>&nbsp; ' + path +
+            ' <button type="button" class="closetab"><i class="fa fa-remove"></i></button>' +
             '</a></li>');
         $('a[href="#' + studio_file_editor_tabs[path] + '"] .closetab', edit_file_tabs).click(function()
         {
@@ -177,6 +176,10 @@ function studio_task_file_open_tab(path)
                           if(mode == undefined)
                           {
                               mode = "text/plain";
+
+                              if(path === "/run") //the default interpreter is IPython
+                                  mode = "python";
+
                               //verify if it is a UNIX executable file that starts with #!
                               if(data['content'].substring(0, 2) == "#!")
                               {
@@ -232,7 +235,7 @@ function studio_task_file_delete_tab(path)
         }
 
         var edit_file_tabs = $('#edit_file_tabs');
-        if($('a[href="#' + studio_file_editor_tabs[path] + '"]', edit_file_tabs).parent().hasClass('active'))
+        if($('a[href="#' + studio_file_editor_tabs[path] + '"]', edit_file_tabs).hasClass('active'))
             $('li:eq(0) a', edit_file_tabs).tab('show');
         $('a[href="#' + studio_file_editor_tabs[path] + '"]', edit_file_tabs).parent().remove();
         $('#' + studio_file_editor_tabs[path]).remove();
@@ -244,9 +247,9 @@ function studio_task_file_delete_tab(path)
 /**
  * Display a message indicating the status of a save action
  */
-function studio_display_task_submit_message(content, type, dismissible)
+function studio_display_task_submit_message(title, content, type, dismissible)
 {
-    var code = getAlertCode(content, type, dismissible);
+    var code = getAlertCode(title, content, type, dismissible);
     $('#task_edit_submit_status').html(code);
 
     if(dismissible)
@@ -271,7 +274,7 @@ function studio_submit()
         return;
     studio_submitting = true;
 
-    studio_display_task_submit_message("Saving...", "info", false);
+    studio_display_task_submit_message("Saving...", "", "info", false);
 
     $('form#edit_task_form .subproblem_order').each(function(index, elem)
     {
@@ -316,9 +319,9 @@ function studio_submit()
     });
 
     if(error)
-        studio_display_task_submit_message("Some error(s) occurred when saving the task: <ul>" + error + "</ul>", "danger", true);
+        studio_display_task_submit_message("Some error(s) occurred when saving the task: <ul>" + error + "</ul>", "", "danger", true);
     else
-        studio_display_task_submit_message("Task saved.", "success", true);
+        studio_display_task_submit_message("Task saved.", "", "success", true);
 
     $('.task_edit_submit_button').attr('disabled', false);
     studio_submitting = false;
@@ -485,8 +488,7 @@ function studio_init_template_multiple_choice(well, pid, problem)
  * @param pid
  * @param choice_data
  */
-function studio_create_choice(pid, choice_data)
-{
+function studio_create_choice(pid, choice_data) {
     var well = $(studio_get_problem(pid));
 
     var index = 0;
@@ -506,13 +508,26 @@ function studio_create_choice(pid, choice_data)
     if("feedback" in choice_data)
         editor_feedback.setValue(choice_data["feedback"]);
 
-    if("valid" in choice_data && choice_data["valid"] == true)
-    {
-        $(".subproblem_multiple_choice_valid", new_row).trigger('click');
-        $(".subproblem_multiple_choice_valid", new_row).attr('checked', true);
+    if("valid" in choice_data && choice_data["valid"] == true) {
+        studio_toggle_choice($(".subproblem_multiple_choice_valid", new_row).attr('name'));
     }
 }
 
+/**
+ * Toggle multiple choice valid field
+ * @param input_name Name of the checkbox input
+ */
+function studio_toggle_choice(input_name) {
+    var checkbox = $("input[name='" + input_name + "']");
+    checkbox.click();
+    var btn = checkbox.next("button");
+    btn.toggleClass("btn-danger");
+    btn.toggleClass("btn-success");
+    var icon = btn.find("i");
+    icon.toggleClass("fa-times");
+    icon.toggleClass("fa-check");
+}
+
 /**
  * Delete a multiple choice answer
  * @param pid
@@ -617,14 +632,6 @@ function studio_get_feedback(sid)
  * Functions for tags edition. Use in tags.html
  */
 
-// Enable or disabled the id field of line {line}
-function studio_change_id_field(type, line) {
-    if(type.value == 2){    
-        $("#id_"+line).prop('disabled', true);
-    }else{
-        $("#id_"+line).prop('disabled', false);
-    }
-}
 function studio_expand_tag_description(elem){
     elem.rows = 5;
 }
@@ -662,12 +669,85 @@ function studio_add_tag_line(line) {
     //TYPE
     var type = $('#E-'+line).attr("data-type");
     modified_row = modified_row.replace("type_replace_"+type, 'selected="selected"');
-    if(type == 2){
-        modified_row = modified_row.replace("id_stop", "disabled");
-    }else{
-        modified_row = modified_row.replace("id_stop", "");
-    }
+    modified_row = modified_row.replace("id_stop", "");
 
     $('#table').find('tbody').append("<tr id="+new_id+">" + modified_row + "</tr>");
     new_row.show();
 }
+
+
+function drag_drop_handler() {
+    // preventing page from redirecting
+    $("html").on("dragover", function(e) {
+        e.preventDefault();
+        e.stopPropagation();
+    });
+
+    $("html").on("drop", function(e) { e.preventDefault(); e.stopPropagation(); });
+
+    // Drag enter
+    $(".upload-area").on('dragenter', function (e) {
+        $("#edit_task_tabs_content").append("<p id='dragtext'><b>Drag a file here</b></p>");
+        e.stopPropagation();
+        e.preventDefault();
+
+    });
+
+    // Drag over
+    $(".upload-area").on('dragover', function (e) {
+        $(this).addClass("dragin");
+        e.stopPropagation();
+        e.preventDefault();
+
+    });
+
+    $(".upload-area").on('dragleave',function(e){
+        $(this).removeClass("dragin");
+        $("#dragtext").remove();
+    });
+
+    // Drop
+    $(".upload-area").on('drop', function (e) {
+        $("#dragtext").remove();
+        e.stopPropagation();
+        e.preventDefault();
+
+        var file = e.originalEvent.dataTransfer.files;
+        var fd = new FormData();
+        fd.append('file', file[0]);
+        fd.append('name',file[0].name);
+        uploadData(fd);
+    });
+
+    // Open file selector on div click
+    $(".upload-area").click(function(){
+        $("#file").click();
+    });
+
+    // file selected
+    $("#file").change(function(){
+        var fd = new FormData();
+        var files = $('#file')[0].files[0];
+        fd.append('file',files);
+        uploadData(fd);
+    });
+}
+
+// Sending AJAX request and upload file
+function uploadData(formdata){
+    $.ajax({
+        url: window.location.href+'/dd_upload',
+        type: 'post',
+        data: formdata,
+        contentType: false,
+        processData: false,
+        dataType: 'json',
+        success: function(response){
+            alert("uploaded!");
+            studio_update_file_tabs(undefined, undefined);
+        },
+        error: function () {
+            console.log("something went wrong");
+        }
+    });
+}
\ No newline at end of file
