@@ -43,7 +43,7 @@ class WebAppSubmissionManager:
         self._logger = logging.getLogger("inginious.webapp.submissions")
         self._lti_outcome_manager = lti_outcome_manager
 
-    def _job_done_callback(self, submissionid, task, result, grade, problems, tests, custom, archive, stdout, stderr, newsub=True):
+    def _job_done_callback(self, submissionid, task, result, grade, problems, tests, custom, state, archive, stdout, stderr, newsub=True):
         """ Callback called by Client when a job is done. Updates the submission in the database with the data returned after the completion of the
         job """
         submission = self.get_submission(submissionid, False)
@@ -59,6 +59,7 @@ class WebAppSubmissionManager:
             "problems": problems,
             "archive": (self._gridfs.put(archive) if archive is not None else None),
             "custom": custom,
+            "state": state,
             "stdout": stdout,
             "stderr": stderr
         }
@@ -80,7 +81,7 @@ class WebAppSubmissionManager:
         self._hook_manager.call_hook("submission_done", submission=submission, archive=archive, newsub=newsub)
 
         for username in submission["username"]:
-            self._user_manager.update_user_stats(username, task, submission, result[0], grade, newsub)
+            self._user_manager.update_user_stats(username, task, submission, result[0], grade, state, newsub)
 
         if "outcome_service_url" in submission and "outcome_result_id" in submission and "outcome_consumer_key" in submission:
             for username in submission["username"]:
@@ -158,9 +159,7 @@ class WebAppSubmissionManager:
             raise Exception("A user must be logged in to submit an object")
 
         # Don't enable ssh debug
-        ssh_callback = lambda host, port, password: None
-        if debug == "ssh":
-            ssh_callback = lambda host, port, password: self._handle_ssh_callback(submission["_id"], host, port, password)
+        ssh_callback = lambda host, port, password: self._handle_ssh_callback(submission["_id"], host, port, password)
 
         # Load input data and add username to dict
         inputdata = bson.BSON.decode(self._gridfs.get(submission["input"]).read())
@@ -177,22 +176,25 @@ class WebAppSubmissionManager:
             username = self._user_manager.session_username()
             submission["username"] = [username]
             submission["submitted_on"] = datetime.now()
+            my_user_task = self._database.user_tasks.find_one({"courseid": task.get_course_id(), "taskid": task.get_id(), "username": username}, { "tried" : 1, "_id" : 0 })
+            tried_count = my_user_task["tried"]
+            inputdata["@attempts"] = str(tried_count + 1)
             inputdata["@username"] = username
             inputdata["@lang"] = self._user_manager.session_language()
             submission["input"] = self._gridfs.put(bson.BSON.encode(inputdata))
             submission["tests"] = {} # Be sure tags are reinitialized
             submissionid = self._database.submissions.insert(submission)
 
-        jobid = self._client.new_job(task, inputdata,
-                                     (lambda result, grade, problems, tests, custom, archive, stdout, stderr:
-                                      self._job_done_callback(submissionid, task, result, grade, problems, tests, custom, archive, stdout, stderr, copy)),
+        jobid = self._client.new_job(1, task, inputdata,
+                                     (lambda result, grade, problems, tests, custom, state, archive, stdout, stderr:
+                                      self._job_done_callback(submissionid, task, result, grade, problems, tests, custom, state, archive, stdout, stderr, copy)),
                                      "Frontend - {}".format(submission["username"]), debug, ssh_callback)
 
         # Clean the submission document in db
         self._database.submissions.update(
             {"_id": submission["_id"]},
             {"$set": {"jobid": jobid, "status": "waiting", "response_type": task.get_response_type()},
-             "$unset": {"result": "", "grade": "", "text": "", "tests": "", "problems": "", "archive": "", "custom": ""}
+             "$unset": {"result": "", "grade": "", "text": "", "tests": "", "problems": "", "archive": "", "state": "", "custom": ""}
              })
 
         if not copy:
@@ -221,7 +223,7 @@ class WebAppSubmissionManager:
         :param inputdata: the input as a dictionary
         :type inputdata: dict
         :param debug: If debug is true, more debug data will be saved
-        :type debug: bool
+        :type debug: bool or string
         :returns: the new submission id and the removed submission id
         """
         if not self._user_manager.session_logged_in():
@@ -252,9 +254,13 @@ class WebAppSubmissionManager:
         # new_submission hook
         inputdata["@username"] = username
         inputdata["@lang"] = self._user_manager.session_language()
+        my_user_task = self._database.user_tasks.find_one({"courseid": task.get_course_id(), "taskid": task.get_id(), "username": username}, { "tried" : 1, "_id" : 0 })
+        tried_count = my_user_task["tried"]
+        inputdata["@attempts"] = str(tried_count + 1)
         # Retrieve input random
-        random_input = self._database.user_tasks.find_one({"courseid": task.get_course_id(), "taskid": task.get_id(), "username": username}, { "random": 1 })
-        inputdata["@random"] = random_input["random"] if "random" in random_input else []
+        states = self._database.user_tasks.find_one({"courseid": task.get_course_id(), "taskid": task.get_id(), "username": username}, {"random": 1, "state": 1})
+        inputdata["@random"] = states["random"] if "random" in states else []
+        inputdata["@state"] = states["state"] if "state" in states else ""
 
         self._hook_manager.call_hook("new_submission", submission=obj, inputdata=inputdata)
         obj["input"] = self._gridfs.put(bson.BSON.encode(inputdata))
@@ -263,13 +269,11 @@ class WebAppSubmissionManager:
         submissionid = self._database.submissions.insert(obj)
         to_remove = self._after_submission_insertion(task, inputdata, debug, obj, submissionid)
 
-        ssh_callback = lambda host, port, password: None
-        if debug == "ssh":
-            ssh_callback = lambda host, port, password: self._handle_ssh_callback(submissionid, host, port, password)
+        ssh_callback = lambda host, port, password: self._handle_ssh_callback(submissionid, host, port, password)
 
-        jobid = self._client.new_job(task, inputdata,
-                                     (lambda result, grade, problems, tests, custom, archive, stdout, stderr:
-                                      self._job_done_callback(submissionid, task, result, grade, problems, tests, custom, archive, stdout, stderr, True)),
+        jobid = self._client.new_job(0, task, inputdata,
+                                     (lambda result, grade, problems, tests, custom, state, archive, stdout, stderr:
+                                      self._job_done_callback(submissionid, task, result, grade, problems, tests, custom, state, archive, stdout, stderr, True)),
                                      "Frontend - {}".format(username), debug, ssh_callback)
 
         self._database.submissions.update(
@@ -483,90 +487,93 @@ class WebAppSubmissionManager:
         """
         tmpfile = archive_file if archive_file is not None else tempfile.TemporaryFile()
         tar = tarfile.open(fileobj=tmpfile, mode='w:gz')
+        error = ""
 
         for submission in submissions:
-            submission = self.get_input_from_submission(submission)
+            try:
+                submission = self.get_input_from_submission(submission)
+                submission_yaml = io.BytesIO(inginious.common.custom_yaml.dump(submission).encode('utf-8'))
 
-            submission_yaml = io.BytesIO(inginious.common.custom_yaml.dump(submission).encode('utf-8'))
-
-            # Considering multiple single submissions for each user
-            for username in submission["username"]:
-                # Compute base path in the tar file
-                base_path = "/"
-                for sub_folder in sub_folders:
-                    if sub_folder == 'taskid':
-                        base_path = submission['taskid'] + base_path
-                    elif sub_folder == 'username':
-                        base_path = '_' + '-'.join(submission['username']) + base_path
-                        base_path = base_path[1:]
-                    elif sub_folder == 'aggregation':
-                        if username in aggregations:
-                            if aggregations[username] is None:
-                                # If classrooms are not used, and user is not grouped, his classroom is replaced by None
-                                base_path = '_' + '-'.join(submission['username']) + base_path
-                                base_path = base_path[1:]
-                            else:
-                                base_path = (aggregations[username]["description"] +
-                                             " (" + str(aggregations[username]["_id"]) + ")").replace(" ", "_") + base_path
-
-                    base_path = '/' + base_path
-                base_path = base_path[1:]
-
-                submission_yaml_fname = base_path + str(submission["_id"]) + '/submission.test'
-
-                # Avoid putting two times the same submission on the same place
-                if submission_yaml_fname not in tar.getnames():
-
-                    info = tarfile.TarInfo(name=submission_yaml_fname)
-                    info.size = submission_yaml.getbuffer().nbytes
-                    info.mtime = time.mktime(submission["submitted_on"].timetuple())
-
-                    # Add file in tar archive
-                    tar.addfile(info, fileobj=submission_yaml)
-
-                    # If there is an archive, add it too
-                    if 'archive' in submission and submission['archive'] is not None and submission['archive'] != "":
-                        subfile = self._gridfs.get(submission['archive'])
-                        subtar = tarfile.open(fileobj=subfile, mode="r:gz")
-
-                        for member in subtar.getmembers():
-                            subtarfile = subtar.extractfile(member)
-                            member.name = base_path + str(submission["_id"]) + "/archive/" + member.name
-                            tar.addfile(member, subtarfile)
-
-                        subtar.close()
-                        subfile.close()
-
-                    # If there files that were uploaded by the student, add them
-                    if submission['input'] is not None:
-                        for pid, problem in submission['input'].items():
-                            # If problem is a dict, it is a file (from the specification of the problems)
-                            if isinstance(problem, dict):
-                                # Get the extension (match extensions with more than one dot too)
-                                DOUBLE_EXTENSIONS = ['.tar.gz', '.tar.bz2', '.tar.bz', '.tar.xz']
-                                ext = ""
-                                if not problem['filename'].endswith(tuple(DOUBLE_EXTENSIONS)):
-                                    _, ext = os.path.splitext(problem['filename'])
+                # Considering multiple single submissions for each user
+                for username in submission["username"]:
+                    # Compute base path in the tar file
+                    base_path = "/"
+                    for sub_folder in sub_folders:
+                        if sub_folder == 'taskid':
+                            base_path = submission['taskid'] + base_path
+                        elif sub_folder == 'username':
+                            base_path = '_' + '-'.join(submission['username']) + base_path
+                            base_path = base_path[1:]
+                        elif sub_folder == 'aggregation':
+                            if username in aggregations:
+                                if aggregations[username] is None:
+                                    # If classrooms are not used, and user is not grouped, his classroom is replaced by None
+                                    base_path = '_' + '-'.join(submission['username']) + base_path
+                                    base_path = base_path[1:]
                                 else:
-                                    for t_ext in DOUBLE_EXTENSIONS:
-                                        if problem['filename'].endswith(t_ext):
-                                            ext = t_ext
+                                    base_path = (aggregations[username]["description"] +
+                                                 " (" + str(aggregations[username]["_id"]) + ")").replace(" ", "_") + base_path
 
-                                subfile = io.BytesIO(problem['value'])
-                                taskfname = base_path + str(submission["_id"]) + '/uploaded_files/' + pid + ext
+                        base_path = '/' + base_path
+                    base_path = base_path[1:]
 
-                                # Generate file info
-                                info = tarfile.TarInfo(name=taskfname)
-                                info.size = subfile.getbuffer().nbytes
-                                info.mtime = time.mktime(submission["submitted_on"].timetuple())
+                    submission_yaml_fname = base_path + str(submission["_id"]) + '/submission.test'
 
-                                # Add file in tar archive
-                                tar.addfile(info, fileobj=subfile)
+                    # Avoid putting two times the same submission on the same place
+                    if submission_yaml_fname not in tar.getnames():
+
+                        info = tarfile.TarInfo(name=submission_yaml_fname)
+                        info.size = submission_yaml.getbuffer().nbytes
+                        info.mtime = time.mktime(submission["submitted_on"].timetuple())
+
+                        # Add file in tar archive
+                        tar.addfile(info, fileobj=submission_yaml)
+
+                        # If there is an archive, add it too
+                        if 'archive' in submission and submission['archive'] is not None and submission['archive'] != "":
+                            subfile = self._gridfs.get(submission['archive'])
+                            subtar = tarfile.open(fileobj=subfile, mode="r:gz")
+
+                            for member in subtar.getmembers():
+                                subtarfile = subtar.extractfile(member)
+                                member.name = base_path + str(submission["_id"]) + "/archive/" + member.name
+                                tar.addfile(member, subtarfile)
+
+                            subtar.close()
+                            subfile.close()
+
+                        # If there files that were uploaded by the student, add them
+                        if submission['input'] is not None:
+                            for pid, problem in submission['input'].items():
+                                if isinstance(problem, dict) and "filename" in problem:
+                                    # Get the extension (match extensions with more than one dot too)
+                                    DOUBLE_EXTENSIONS = ['.tar.gz', '.tar.bz2', '.tar.bz', '.tar.xz']
+                                    ext = ""
+                                    if not problem['filename'].endswith(tuple(DOUBLE_EXTENSIONS)):
+                                        _, ext = os.path.splitext(problem['filename'])
+                                    else:
+                                        for t_ext in DOUBLE_EXTENSIONS:
+                                            if problem['filename'].endswith(t_ext):
+                                                ext = t_ext
+
+                                    subfile = io.BytesIO(problem['value'])
+                                    taskfname = base_path + str(submission["_id"]) + '/uploaded_files/' + pid + ext
+
+                                    # Generate file info
+                                    info = tarfile.TarInfo(name=taskfname)
+                                    info.size = subfile.getbuffer().nbytes
+                                    info.mtime = time.mktime(submission["submitted_on"].timetuple())
+
+                                    # Add file in tar archive
+                                    tar.addfile(info, fileobj=subfile)
+            except Exception as e:
+                error = str(submission["_id"])
+                break
 
         # Close tarfile and put tempfile cursor at 0
         tar.close()
         tmpfile.seek(0)
-        return tmpfile
+        return tmpfile, error
 
     def _handle_ssh_callback(self, submission_id, host, port, password):
         """ Handles the creation of a remote ssh server """
